<?php

namespace Drupal\revision_graph;

use Drupal\Core\Database\Database;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\node\NodeInterface;

class RevisionGraphStorage {

  const UNTRACKED_BANCH_NAME = 'untracked';

  public function initGraph( ContentEntityInterface $entity) {
    $conn = Database::getConnection();
    $conn->insert('revision_graph')->fields(
      array(
        'primary_id' => $entity->id(),
        'version_id' => $entity->getRevisionId(),
        'parent_version_id' => $entity->getRevisionId(),
        'type' => $entity->getEntityTypeId(),
        'branch' => $this->formatBranchName($entity->getRevisionId(), $entity->language()->getId()),
      )
    )->execute();
  }

  public function updateGraph( ContentEntityInterface $entity) {
    $conn = Database::getConnection();
    // @TODO hunt actual parent version id.
    $parent_version_id = $entity->original->getRevisionId();
    $branch = $this->getParentBranchName($parent_version_id);

    // For some reason we have default revision id in original.
    // Hackish way to sniff the actual parent.
    // @TODO Check if this is a bug in core due to some optimisation.
    $current_path = \Drupal::service('path.current')->getPath();
    $path_args = explode('/', $current_path);
    if ($path_args[1] == 'node' && $path_args[3] == 'revisions' && $path_args[5] == 'revert') {
      $parent_version_id = (int) $path_args[4];
      $branch = $parent_version_id;
    }


    $conn->insert('revision_graph')->fields(
      array(
        'primary_id' => $entity->id(),
        'version_id' => $entity->getRevisionId(),
        'parent_version_id' => $parent_version_id,
        'type' => $entity->getEntityTypeId(),
        'branch' => $this->formatBranchName($branch, $entity->language()->getId()),
      )
    )->execute();
  }
  private function formatBranchName($parent_branch, $language_id) {
    list($branch, $code) = explode(':', $parent_branch);
    return implode(':', [$branch, $language_id]);
  }

  private function getParentBranchName($version_id) {
    $connection = \Drupal::service('database');
    $result = $connection->query("SELECT * FROM {revision_graph} WHERE version_id = :vid", [':vid' => $version_id])->fetchAll();
    $branch_name = self::UNTRACKED_BANCH_NAME;
    foreach ($result as $r) {
      $branch_name = $r->branch;
      break;
    }
    return $branch_name;
  }
}
